@@ -31,6 +31,20 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 
+static const struct machine_dir
+{
+    DWORD machine;
+    const WCHAR *pe_dir;
+    const WCHAR *so_dir;
+}
+    all_machine_dir[] =
+{
+    {IMAGE_FILE_MACHINE_I386,  L"\\i386-windows\\",    L"\\i386-unix\\"},
+    {IMAGE_FILE_MACHINE_AMD64, L"\\x86_64-windows\\",  L"\\x86_64-unix\\"},
+    {IMAGE_FILE_MACHINE_ARMNT, L"\\arm-windows\\",     L"\\arm-unix\\"},
+    {IMAGE_FILE_MACHINE_ARM64, L"\\aarch64-windows\\", L"\\aarch64-unix\\"},
+};
+
 static inline BOOL is_sepA(char ch) {return ch == '/' || ch == '\\';}
 static inline BOOL is_sep(WCHAR ch) {return ch == '/' || ch == '\\';}
 
@@ -225,14 +239,11 @@ static BOOL do_searchW(PCWSTR file, PWSTR buffer, BOOL recurse,
     WIN32_FIND_DATAW    fd;
     unsigned            pos;
     BOOL                found = FALSE;
-    static const WCHAR  S_AllW[] = {'*','.','*','\0'};
-    static const WCHAR  S_DotW[] = {'.','\0'};
-    static const WCHAR  S_DotDotW[] = {'.','.','\0'};
 
     pos = lstrlenW(buffer);
     if (pos == 0) return FALSE;
     if (buffer[pos - 1] != '\\') buffer[pos++] = '\\';
-    lstrcpyW(buffer + pos, S_AllW);
+    lstrcpyW(buffer + pos, L"*.*");
     if ((h = FindFirstFileW(buffer, &fd)) == INVALID_HANDLE_VALUE)
         return FALSE;
     /* doc doesn't specify how the tree is enumerated...
@@ -240,7 +251,7 @@ static BOOL do_searchW(PCWSTR file, PWSTR buffer, BOOL recurse,
      */
     do
     {
-        if (!wcscmp(fd.cFileName, S_DotW) || !wcscmp(fd.cFileName, S_DotDotW)) continue;
+        if (!wcscmp(fd.cFileName, L".") || !wcscmp(fd.cFileName, L"..")) continue;
 
         lstrcpyW(buffer + pos, fd.cFileName);
         if (recurse && (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
@@ -375,7 +386,7 @@ BOOL WINAPI SymFindFileInPathW(HANDLE hProcess, PCWSTR searchPath, PCWSTR full_p
     WCHAR*              ptr;
     const WCHAR*        filename;
 
-    TRACE("(hProcess = %p, searchPath = %s, full_path = %s, id = %p, two = 0x%08x, three = 0x%08x, flags = 0x%08x, buffer = %p, cb = %p, user = %p)\n",
+    TRACE("(hProcess = %p, searchPath = %s, full_path = %s, id = %p, two = 0x%08lx, three = 0x%08lx, flags = 0x%08lx, buffer = %p, cb = %p, user = %p)\n",
           hProcess, debugstr_w(searchPath), debugstr_w(full_path),
           id, two, three, flags, buffer, cb, user);
 
@@ -603,7 +614,7 @@ BOOL path_find_symbol_file(const struct process* pcs, const struct module* modul
     const WCHAR*        filename;
     WCHAR*              searchPath = pcs->search_path;
 
-    TRACE("(pcs = %p, full_path = %s, guid = %s, dw1 = 0x%08x, dw2 = 0x%08x, buffer = %p)\n",
+    TRACE("(pcs = %p, full_path = %s, guid = %s, dw1 = 0x%08lx, dw2 = 0x%08lx, buffer = %p)\n",
           pcs, debugstr_a(full_path), debugstr_guid(guid), dw1, dw2, buffer);
 
     mf.guid = guid;
@@ -691,61 +702,82 @@ WCHAR *get_dos_file_name(const WCHAR *filename)
     return dos_path;
 }
 
+static inline const WCHAR* get_machine_dir(const struct machine_dir *machine_dir, const WCHAR *name)
+{
+    WCHAR *ptr;
+    if ((ptr = wcsrchr(name, L'.')) && !lstrcmpW(ptr, L".so"))
+        return machine_dir->so_dir;
+    return machine_dir->pe_dir;
+}
+
+static BOOL try_match_file(const WCHAR *name, BOOL (*match)(void*, HANDLE, const WCHAR*), void *param)
+{
+    HANDLE file = CreateFileW(name, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+    if (file != INVALID_HANDLE_VALUE)
+    {
+        BOOL ret = match(param, file, name);
+        CloseHandle(file);
+        return ret;
+    }
+    return FALSE;
+}
+
 BOOL search_dll_path(const struct process *process, const WCHAR *name, BOOL (*match)(void*, HANDLE, const WCHAR*), void *param)
 {
     const WCHAR *env;
-    size_t len, i;
-    HANDLE file;
+    WCHAR *p, *end;
+    size_t len, i, machine_dir_len;
     WCHAR *buf;
-    BOOL ret;
+    const struct cpu* cpu;
+    const struct machine_dir* machine_dir;
 
     name = file_name(name);
 
+    cpu = process_get_cpu(process);
+    for (machine_dir = all_machine_dir; machine_dir < all_machine_dir + ARRAY_SIZE(all_machine_dir); machine_dir++)
+        if (machine_dir->machine == cpu->machine) break;
+    if (machine_dir >= all_machine_dir + ARRAY_SIZE(all_machine_dir)) return FALSE;
+    machine_dir_len = max(wcslen(machine_dir->pe_dir), wcslen(machine_dir->so_dir));
+
     if ((env = process_getenv(process, L"WINEBUILDDIR")))
     {
-        WCHAR *p, *end;
         const WCHAR dllsW[] = { '\\','d','l','l','s','\\' };
         const WCHAR programsW[] = { '\\','p','r','o','g','r','a','m','s','\\' };
-        const WCHAR dot_dllW[] = {'.','d','l','l',0};
-        const WCHAR dot_exeW[] = {'.','e','x','e',0};
-        const WCHAR dot_soW[] = {'.','s','o',0};
-
 
         len = lstrlenW(env);
-        if (!(buf = heap_alloc((len + 8 + 3 * lstrlenW(name)) * sizeof(WCHAR)))) return FALSE;
+        if (!(buf = heap_alloc((len + ARRAY_SIZE(programsW) + machine_dir_len +
+                                2 * lstrlenW(name) + 1) * sizeof(WCHAR)))) return FALSE;
         wcscpy(buf, env);
         end = buf + len;
 
         memcpy(end, dllsW, sizeof(dllsW));
         lstrcpyW(end + ARRAY_SIZE(dllsW), name);
-        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, dot_soW)) *p = 0;
-        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, dot_dllW)) *p = 0;
+        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, L".so")) *p = 0;
+        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, L".dll")) *p = 0;
         p = end + lstrlenW(end);
+        /* try multi-arch first */
+        wcscpy(p, get_machine_dir(machine_dir, name));
+        wcscpy(p + wcslen(p), name);
+        if (try_match_file(buf, match, param)) goto found;
+        /* then old mono-arch */
         *p++ = '\\';
         lstrcpyW(p, name);
-        file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (file != INVALID_HANDLE_VALUE)
-        {
-            ret = match(param, file, buf);
-            CloseHandle(file);
-            if (ret) goto found;
-        }
+        if (try_match_file(buf, match, param)) goto found;
 
         memcpy(end, programsW, sizeof(programsW));
         end += ARRAY_SIZE(programsW);
         lstrcpyW(end, name);
-        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, dot_soW)) *p = 0;
-        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, dot_exeW)) *p = 0;
+        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, L".so")) *p = 0;
+        if ((p = wcsrchr(end, '.')) && !lstrcmpW(p, L".exe")) *p = 0;
         p = end + lstrlenW(end);
+        /* try multi-arch first */
+        wcscpy(p, get_machine_dir(machine_dir, name));
+        wcscpy(p + wcslen(p), name);
+        if (try_match_file(buf, match, param)) goto found;
+        /* then old mono-arch */
         *p++ = '\\';
         lstrcpyW(p, name);
-        file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (file != INVALID_HANDLE_VALUE)
-        {
-            ret = match(param, file, buf);
-            CloseHandle(file);
-            if (ret) goto found;
-        }
+        if (try_match_file(buf, match, param)) goto found;
 
         heap_free(buf);
     }
@@ -755,16 +787,12 @@ BOOL search_dll_path(const struct process *process, const WCHAR *name, BOOL (*ma
         WCHAR env_name[64];
         swprintf(env_name, ARRAY_SIZE(env_name), L"WINEDLLDIR%u", i);
         if (!(env = process_getenv(process, env_name))) return FALSE;
-        len = wcslen(env) + wcslen(name) + 2;
+        len = wcslen(env) + machine_dir_len + wcslen(name) + 1;
         if (!(buf = heap_alloc(len * sizeof(WCHAR)))) return FALSE;
+        swprintf(buf, len, L"%s%s%s", env, get_machine_dir(machine_dir, name), name);
+        if (try_match_file(buf, match, param)) goto found;
         swprintf(buf, len, L"%s\\%s", env, name);
-        file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (file != INVALID_HANDLE_VALUE)
-        {
-            ret = match(param, file, buf);
-            CloseHandle(file);
-            if (ret) goto found;
-        }
+        if (try_match_file(buf, match, param)) goto found;
         heap_free(buf);
     }
 
@@ -801,13 +829,8 @@ BOOL search_unix_path(const WCHAR *name, const WCHAR *path, BOOL (*match)(void*,
             WideCharToMultiByte(CP_UNIXCP, 0, name, -1, buf + len, size - len, NULL, NULL);
             if ((dos_path = wine_get_dos_file_name(buf)))
             {
-                HANDLE file = CreateFileW(dos_path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-                if (file != INVALID_HANDLE_VALUE)
-                {
-                    ret = match(param, file, dos_path);
-                    CloseHandle(file);
-                    if (ret) TRACE("found %s\n", debugstr_w(dos_path));
-                }
+                ret = try_match_file(dos_path, match, param);
+                if (ret) TRACE("found %s\n", debugstr_w(dos_path));
                 heap_free(dos_path);
                 if (ret) break;
             }
