@@ -314,23 +314,27 @@ struct symt_data* symt_new_global_variable(struct module* module,
     return sym;
 }
 
-static void init_function_or_inlinesite(struct symt_function* sym,
-                                        struct module* module,
-                                        DWORD tag,
-                                        struct symt* container,
-                                        const char* name,
-                                        ULONG_PTR addr, ULONG_PTR size,
-                                        struct symt* sig_type)
+static struct symt_function* init_function_or_inlinesite(struct module* module,
+                                                         DWORD tag,
+                                                         struct symt* container,
+                                                         const char* name,
+                                                         struct symt* sig_type,
+                                                         unsigned num_ranges)
 {
+    struct symt_function* sym;
+
     assert(!sig_type || sig_type->tag == SymTagFunctionType);
-    sym->symt.tag  = tag;
-    sym->hash_elt.name = pool_strdup(&module->pool, name);
-    sym->container = container;
-    sym->address   = addr;
-    sym->type      = sig_type;
-    sym->size      = size;
-    vector_init(&sym->vlines,  sizeof(struct line_info), 64);
-    vector_init(&sym->vchildren, sizeof(struct symt*), 8);
+    if ((sym = pool_alloc(&module->pool, offsetof(struct symt_function, ranges[num_ranges]))))
+    {
+        sym->symt.tag  = tag;
+        sym->hash_elt.name = pool_strdup(&module->pool, name);
+        sym->container = container;
+        sym->type      = sig_type;
+        vector_init(&sym->vlines,  sizeof(struct line_info), 64);
+        vector_init(&sym->vchildren, sizeof(struct symt*), 8);
+        sym->num_ranges = num_ranges;
+    }
+    return sym;
 }
 
 struct symt_function* symt_new_function(struct module* module,
@@ -343,10 +347,11 @@ struct symt_function* symt_new_function(struct module* module,
 
     TRACE_(dbghelp_symt)("Adding global function %s:%s @%Ix-%Ix\n",
                          debugstr_w(module->modulename), name, addr, addr + size - 1);
-    if ((sym = pool_alloc(&module->pool, sizeof(*sym))))
+    if ((sym = init_function_or_inlinesite(module, SymTagFunction, &compiland->symt, name, sig_type, 1)))
     {
         struct symt** p;
-        init_function_or_inlinesite(sym, module, SymTagFunction, &compiland->symt, name, addr, size, sig_type);
+        sym->ranges[0].low = addr;
+        sym->ranges[0].high = addr + size;
         sym->next_inlinesite = NULL; /* first of list */
         symt_add_module_ht(module, (struct symt_ht*)sym);
         if (compiland)
@@ -358,24 +363,23 @@ struct symt_function* symt_new_function(struct module* module,
     return sym;
 }
 
-struct symt_inlinesite* symt_new_inlinesite(struct module* module,
-                                            struct symt_function* func,
-                                            struct symt* container,
-                                            const char* name,
-                                            ULONG_PTR addr,
-                                            struct symt* sig_type)
+struct symt_function* symt_new_inlinesite(struct module* module,
+                                          struct symt_function* func,
+                                          struct symt* container,
+                                          const char* name,
+                                          struct symt* sig_type,
+                                          unsigned num_ranges)
 {
-    struct symt_inlinesite* sym;
+    struct symt_function* sym;
 
-    TRACE_(dbghelp_symt)("Adding inline site %s @%Ix\n", name, addr);
-    if ((sym = pool_alloc(&module->pool, sizeof(*sym))))
+    TRACE_(dbghelp_symt)("Adding inline site %s\n", name);
+    if ((sym = init_function_or_inlinesite(module, SymTagInlineSite, container, name, sig_type, num_ranges)))
     {
         struct symt** p;
         assert(container);
-        init_function_or_inlinesite(&sym->func, module, SymTagInlineSite, container, name, addr, 0, sig_type);
-        vector_init(&sym->vranges, sizeof(struct addr_range), 2); /* FIXME: number of elts => to be set on input */
+
         /* chain inline sites */
-        sym->func.next_inlinesite = func->next_inlinesite;
+        sym->next_inlinesite = func->next_inlinesite;
         func->next_inlinesite = sym;
         if (container->tag == SymTagFunction || container->tag == SymTagInlineSite)
             p = vector_add(&((struct symt_function*)container)->vchildren, &module->pool);
@@ -384,7 +388,7 @@ struct symt_inlinesite* symt_new_inlinesite(struct module* module,
             assert(container->tag == SymTagBlock);
             p = vector_add(&((struct symt_block*)container)->vchildren, &module->pool);
         }
-        *p = &sym->func.symt;
+        *p = &sym->symt;
     }
     return sym;
 }
@@ -535,7 +539,7 @@ struct symt_block* symt_open_func_block(struct module* module,
     assert(num_ranges > 0);
     assert(!parent_block || parent_block->symt.tag == SymTagBlock);
 
-    block = pool_alloc(&module->pool, sizeof(*block) + num_ranges * sizeof(block->ranges[0]));
+    block = pool_alloc(&module->pool, offsetof(struct symt_block, ranges[num_ranges]));
     block->symt.tag = SymTagBlock;
     block->num_ranges = num_ranges;
     block->container = parent_block ? &parent_block->symt : &func->symt;
@@ -580,42 +584,6 @@ struct symt_hierarchy_point* symt_add_function_point(struct module* module,
     return sym;
 }
 
-/* low and high are absolute addresses */
-BOOL symt_add_inlinesite_range(struct module* module,
-                               struct symt_inlinesite* inlined,
-                               ULONG_PTR low, ULONG_PTR high)
-{
-    struct addr_range* p;
-
-    p = vector_add(&inlined->vranges, &module->pool);
-    p->low = low;
-    p->high = high;
-    if (TRUE)
-    {
-        int i;
-
-        /* see dbghelp_private.h for the assumptions */
-        for (i = 0; i < inlined->vranges.num_elts - 1; i++)
-        {
-            if (!addr_range_disjoint((struct addr_range*)vector_at(&inlined->vranges, i), p))
-            {
-                FIXME("Added addr_range isn't disjoint from siblings\n");
-            }
-        }
-        for ( ; inlined->func.symt.tag != SymTagFunction; inlined = (struct symt_inlinesite*)symt_get_upper_inlined(inlined))
-        {
-            for (i = 0; i < inlined->vranges.num_elts; i++)
-            {
-                struct addr_range* ar = (struct addr_range*)vector_at(&inlined->vranges, i);
-                if (!addr_range_disjoint(ar, p) && !addr_range_inside(ar, p))
-                    WARN("Added addr_range not compatible with parent\n");
-            }
-        }
-    }
-
-    return TRUE;
-}
-
 struct symt_thunk* symt_new_thunk(struct module* module, 
                                   struct symt_compiland* compiland, 
                                   const char* name, THUNK_ORDINAL ord,
@@ -1237,19 +1205,18 @@ void copy_symbolW(SYMBOL_INFOW* siw, const SYMBOL_INFO* si)
 }
 
 /* return the lowest inline site inside a function */
-struct symt_inlinesite* symt_find_lowest_inlined(struct symt_function* func, DWORD64 addr)
+struct symt_function* symt_find_lowest_inlined(struct symt_function* func, DWORD64 addr)
 {
-    struct symt_inlinesite* current;
+    struct symt_function* current;
     int i;
 
     assert(func->symt.tag == SymTagFunction);
-    for (current = func->next_inlinesite; current; current = current->func.next_inlinesite)
+    for (current = func->next_inlinesite; current; current = current->next_inlinesite)
     {
-        for (i = 0; i < current->vranges.num_elts; ++i)
+        for (i = 0; i < current->num_ranges; ++i)
         {
-            struct addr_range* ar = (struct addr_range*)vector_at(&current->vranges, i);
             /* first matching range gives the lowest inline site; see dbghelp_private.h for details */
-            if (ar->low <= addr && addr < ar->high)
+            if (current->ranges[i].low <= addr && addr < current->ranges[i].high)
                 return current;
         }
     }
@@ -1257,9 +1224,9 @@ struct symt_inlinesite* symt_find_lowest_inlined(struct symt_function* func, DWO
 }
 
 /* from an inline function, get either the enclosing inlined function, or the top function when no inlined */
-struct symt* symt_get_upper_inlined(struct symt_inlinesite* inlined)
+struct symt* symt_get_upper_inlined(struct symt_function* inlined)
 {
-    struct symt* symt = &inlined->func.symt;
+    struct symt* symt = &inlined->symt;
 
     do
     {
@@ -1274,18 +1241,18 @@ struct symt* symt_get_upper_inlined(struct symt_inlinesite* inlined)
 }
 
 /* lookup in module for an inline site (from addr and inline_ctx) */
-struct symt_inlinesite* symt_find_inlined_site(struct module* module, DWORD64 addr, DWORD inline_ctx)
+struct symt_function* symt_find_inlined_site(struct module* module, DWORD64 addr, DWORD inline_ctx)
 {
     struct symt_ht* symt = symt_find_symbol_at(module, addr);
 
     if (symt_check_tag(&symt->symt, SymTagFunction))
     {
         struct symt_function* func = (struct symt_function*)symt;
-        struct symt_inlinesite* curr = symt_find_lowest_inlined(func, addr);
+        struct symt_function* curr = symt_find_lowest_inlined(func, addr);
         DWORD depth = IFC_DEPTH(inline_ctx);
 
         if (curr)
-            for ( ; &curr->func != func; curr = (struct symt_inlinesite*)symt_get_upper_inlined(curr))
+            for ( ; curr != func; curr = (struct symt_function*)symt_get_upper_inlined(curr))
                 if (depth-- == 0) return curr;
     }
     return NULL;
@@ -1301,10 +1268,10 @@ DWORD symt_get_inlinesite_depth(HANDLE hProcess, DWORD64 addr)
         struct symt_ht* symt = symt_find_symbol_at(pair.effective, addr);
         if (symt_check_tag(&symt->symt, SymTagFunction))
         {
-            struct symt_inlinesite* inlined = symt_find_lowest_inlined((struct symt_function*)symt, addr);
+            struct symt_function* inlined = symt_find_lowest_inlined((struct symt_function*)symt, addr);
             if (inlined)
             {
-                for ( ; &inlined->func.symt != &symt->symt; inlined = (struct symt_inlinesite*)symt_get_upper_inlined(inlined))
+                for ( ; &inlined->symt != &symt->symt; inlined = (struct symt_function*)symt_get_upper_inlined(inlined))
                     ++depth;
             }
         }
@@ -2710,7 +2677,7 @@ PWSTR WINAPI SymSetHomeDirectoryW(HANDLE hProcess, PCWSTR dir)
 BOOL WINAPI SymFromInlineContext(HANDLE hProcess, DWORD64 addr, ULONG inline_ctx, PDWORD64 disp, PSYMBOL_INFO si)
 {
     struct module_pair pair;
-    struct symt_inlinesite* inlined;
+    struct symt_function* inlined;
 
     TRACE("(%p, %#I64x, 0x%lx, %p, %p)\n", hProcess, addr, inline_ctx, disp, si);
 
@@ -2721,8 +2688,8 @@ BOOL WINAPI SymFromInlineContext(HANDLE hProcess, DWORD64 addr, ULONG inline_ctx
         inlined = symt_find_inlined_site(pair.effective, addr, inline_ctx);
         if (inlined)
         {
-            symt_fill_sym_info(&pair, NULL, &inlined->func.symt, si);
-            if (disp) *disp = addr - inlined->func.address;
+            symt_fill_sym_info(&pair, NULL, &inlined->symt, si);
+            if (disp) *disp = addr - inlined->ranges[0].low;
             return TRUE;
         }
         /* fall through */
@@ -2765,14 +2732,14 @@ static BOOL get_line_from_inline_context(HANDLE hProcess, DWORD64 addr, ULONG in
                                          struct internal_line_t* intl)
 {
     struct module_pair pair;
-    struct symt_inlinesite* inlined;
+    struct symt_function* inlined;
 
     if (!module_init_pair(&pair, hProcess, mod_addr ? mod_addr : addr)) return FALSE;
     switch (IFC_MODE(inline_ctx))
     {
     case IFC_MODE_INLINE:
         inlined = symt_find_inlined_site(pair.effective, addr, inline_ctx);
-        if (inlined && get_line_from_function(&pair, &inlined->func, addr, disp, intl))
+        if (inlined && get_line_from_function(&pair, inlined, addr, disp, intl))
             return TRUE;
         /* fall through: check if we can find line info at top function level */
     case IFC_MODE_IGNORE:
